﻿using ProjectLinkor.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectLinkor.Drawnings;

public class DrawingShip
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityShip? EntityShip { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки корабля
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки корабля
    /// </summary>
    protected int? _startPosY;
    /// <summary>
    /// Ширина прорисовки корабля
    /// </summary>
    private readonly int _drawningShipWidth = 95;
    /// <summary>
    /// Высота прорисовки корабля
    /// </summary>
    private readonly int _drawningShipHeight = 40;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;
    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;
    // <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningShipWidth;
    // <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningShipHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawingShip()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawingShip(int speed, double weight, Color bodyColor) : this() 
    {
        EntityShip = new EntityShip(speed, weight, bodyColor);
    }
    /// <summary>
    /// Конструктор для наследоваников
    /// </summary>
    /// <param name="drawningLinkorWidth">Ширина прорисовки корабля</param>
    /// <param name="drawningLinkorHeight">Высота прорисовки корабля</param>
    protected DrawingShip(int drawningLinkorWidth, int drawningLinkorHeight) : this()
    {
        _drawningShipWidth = drawningLinkorWidth;
        _drawningShipHeight = drawningLinkorHeight;
    }

    public DrawingShip(EntityShip? ship) : this()
    {
        if (ship == null) return;
        EntityShip = new EntityShip(ship.Speed, ship.Weight, ship.BodyColor);
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена

        if (width > _drawningShipWidth && height > _drawningShipHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;

            if (_startPosX != null && _startPosY != null)
            {
                if (_startPosX.Value < 0) _startPosX = 0;
                if (_startPosY.Value < 0) _startPosY = 0;
                if (_startPosX.Value + _drawningShipWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningShipWidth;
                }
                if (_startPosY.Value + _drawningShipHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningShipHeight;
                }
            }

            return true;
        }
        return false;

    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        else
        {
            _startPosX = x;
            _startPosY = y;

            if (_startPosX.Value < 0) _startPosX = 0;
            if (_startPosY.Value < 0) _startPosY = 0;
            if (_startPosX.Value + _drawningShipWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningShipWidth;
            }
            if (_startPosY.Value + _drawningShipHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningShipHeight;
            }

        }


    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityShip == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityShip.Step > 0)
                {
                    _startPosX -= (int)EntityShip.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityShip.Step > 0)
                {
                    _startPosY -= (int)EntityShip.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningShipWidth + EntityShip.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityShip.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningShipHeight + EntityShip.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityShip.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityShip == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);

        Point[] body = { new Point(_startPosX.Value + 65, _startPosY.Value),
                new Point(_startPosX.Value + 95, _startPosY.Value + 20), new Point(_startPosX.Value +65, _startPosY.Value + 40) };
        


        //границы Линкора
        g.DrawPolygon(pen, body);
        g.DrawRectangle(pen, _startPosX.Value+ 5, _startPosY.Value, 60, 40);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 5, 5, 10);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 25, 5, 10);



        //корпус
        Brush br = new SolidBrush(EntityShip.BodyColor);
        g.FillPolygon(br, body);
        g.FillRectangle(br, _startPosX.Value + 5, _startPosY.Value, 60, 40);
        g.FillEllipse(br, _startPosX.Value + 60, _startPosY.Value + 15, 10, 10);

        //Мачта
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillRectangle(brBlack, _startPosX.Value + 45, _startPosY.Value + 10, 10, 20);
        g.FillRectangle(brBlack, _startPosX.Value + 25, _startPosY.Value + 15, 20, 10);

        //задние трубы
        g.FillRectangle(brBlack, _startPosX.Value, _startPosY.Value + 5, 5, 10);
        g.FillRectangle(brBlack, _startPosX.Value, _startPosY.Value + 25, 5, 10);

    }
}
